"use strict";

var _interopRequireWildcard = require("@babel/runtime/helpers/interopRequireWildcard");

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

exports.__esModule = true;
exports.default = void 0;

var _extends2 = _interopRequireDefault(require("@babel/runtime/helpers/extends"));

var _assertThisInitialized2 = _interopRequireDefault(require("@babel/runtime/helpers/assertThisInitialized"));

var _inheritsLoose2 = _interopRequireDefault(require("@babel/runtime/helpers/inheritsLoose"));

var React = _interopRequireWildcard(require("react"));

var _get = _interopRequireDefault(require("lodash/get"));

var _pick = _interopRequireDefault(require("lodash/pick"));

var _domLib = require("dom-lib");

var _Overlay = _interopRequireDefault(require("./Overlay"));

var _createChainedFunction = _interopRequireDefault(require("../utils/createChainedFunction"));

var _isOneOf = _interopRequireDefault(require("../utils/isOneOf"));

var _getDOMNode = _interopRequireDefault(require("../utils/getDOMNode"));

var _Portal = _interopRequireDefault(require("../Portal"));

function isNullOrUndefined(value) {
  return value === null || typeof value === 'undefined';
}

function onMouseEventHandler(handler, event) {
  var target = event.currentTarget;
  var related = event.relatedTarget || (0, _get.default)(event, ['nativeEvent', 'toElement']);

  if ((!related || related !== target) && !(0, _domLib.contains)(target, related)) {
    handler(event);
  }
}

var OverlayTrigger =
/*#__PURE__*/
function (_React$Component) {
  (0, _inheritsLoose2.default)(OverlayTrigger, _React$Component);

  function OverlayTrigger(props) {
    var _this;

    _this = _React$Component.call(this, props) || this;
    _this.onMouseOverListener = void 0;
    _this.onMouseOutListener = void 0;
    _this.hoverShowDelayTimer = void 0;
    _this.hoverHideDelayTimer = void 0;
    _this.mouseEnteredToSpeaker = false;
    _this.mouseEnteredToTrigger = false;

    _this.getOverlayTarget = function () {
      return (0, _getDOMNode.default)((0, _assertThisInitialized2.default)(_this));
    };

    _this.handleSpeakerMouseEnter = function () {
      _this.mouseEnteredToSpeaker = true;
    };

    _this.handleSpeakerMouseLeave = function () {
      var trigger = _this.props.trigger;
      _this.mouseEnteredToSpeaker = false;

      if (!(0, _isOneOf.default)('click', trigger) && !(0, _isOneOf.default)('active', trigger)) {
        _this.handleHide();
      }
    };

    _this.hide = function () {
      _this.setState({
        isOverlayShown: false
      });
    };

    _this.show = function () {
      _this.setState({
        isOverlayShown: true
      });
    };

    _this.handleHide = function () {
      if (!_this.mouseEnteredToSpeaker && !_this.mouseEnteredToTrigger) {
        _this.hide();
      }
    };

    _this.handleToggle = function () {
      if (_this.state.isOverlayShown) {
        _this.handleHide();
      } else {
        _this.show();
      }
    };

    _this.handleDelayedShow = function () {
      var _this$props = _this.props,
          delayShow = _this$props.delayShow,
          delay = _this$props.delay,
          enterable = _this$props.enterable;

      if (!enterable) {
        _this.show();

        return;
      }

      _this.mouseEnteredToTrigger = true;

      if (!isNullOrUndefined(_this.hoverHideDelayTimer)) {
        clearTimeout(_this.hoverHideDelayTimer);
        _this.hoverHideDelayTimer = null;

        _this.show();

        return;
      }

      if (_this.state.isOverlayShown) {
        return;
      }

      var nextDelay = !isNullOrUndefined(delayShow) ? delayShow : delay;

      if (!nextDelay) {
        _this.show();

        return;
      }

      _this.hoverShowDelayTimer = setTimeout(function () {
        _this.hoverShowDelayTimer = null;

        _this.show();
      }, nextDelay);
    };

    _this.handleDelayedHide = function () {
      var _this$props2 = _this.props,
          delayHide = _this$props2.delayHide,
          delay = _this$props2.delay,
          enterable = _this$props2.enterable;

      if (!enterable) {
        _this.hide();

        return;
      }

      _this.mouseEnteredToTrigger = false;

      if (!isNullOrUndefined(_this.hoverShowDelayTimer)) {
        clearTimeout(_this.hoverShowDelayTimer);
        _this.hoverShowDelayTimer = null;
        return;
      }

      if (!_this.state.isOverlayShown || !isNullOrUndefined(_this.hoverHideDelayTimer)) {
        return;
      }

      var nextDelay = !isNullOrUndefined(delayHide) ? delayHide : delay;

      if (!nextDelay) {
        _this.handleHide();

        return;
      }

      _this.hoverHideDelayTimer = setTimeout(function () {
        if (_this.state.isOnSpeaker) {
          return;
        }

        clearTimeout(_this.hoverHideDelayTimer);
        _this.hoverHideDelayTimer = null;

        _this.handleHide();
      }, nextDelay);
    };

    _this.onMouseOverListener = function (e) {
      return onMouseEventHandler(_this.handleDelayedShow, e);
    };

    _this.onMouseOutListener = function (e) {
      return onMouseEventHandler(_this.handleDelayedHide, e);
    };

    _this.state = {
      isOverlayShown: props.defaultOpen
    };
    return _this;
  }

  var _proto = OverlayTrigger.prototype;

  _proto.componentWillUnmount = function componentWillUnmount() {
    clearTimeout(this.hoverShowDelayTimer);
    clearTimeout(this.hoverHideDelayTimer);
  };

  _proto.getOverlay = function getOverlay() {
    var _this$props3 = this.props,
        open = _this$props3.open,
        speaker = _this$props3.speaker,
        trigger = _this$props3.trigger,
        onHide = _this$props3.onHide;
    var isOverlayShown = this.state.isOverlayShown;
    var overlayProps = (0, _extends2.default)({}, (0, _pick.default)(this.props, Object.keys(_Overlay.default.propTypes)), {
      show: typeof open === 'undefined' ? isOverlayShown : open,
      target: this.getOverlayTarget
    });

    if ((0, _isOneOf.default)('click', trigger)) {
      overlayProps.onHide = (0, _createChainedFunction.default)(this.hide, onHide);
    } else if ((0, _isOneOf.default)('active', trigger)) {
      overlayProps.onHide = (0, _createChainedFunction.default)(this.hide, onHide);
    }

    var speakerProps = {
      onMouseEnter: this.handleSpeakerMouseEnter,
      onMouseLeave: this.handleSpeakerMouseLeave,
      placement: overlayProps.placement
    };

    if (typeof speaker === 'function') {
      return React.createElement(_Overlay.default, overlayProps, speaker);
    }

    return React.createElement(_Overlay.default, overlayProps, React.cloneElement(speaker, speakerProps));
  };

  _proto.render = function render() {
    var _this$props4 = this.props,
        children = _this$props4.children,
        speaker = _this$props4.speaker,
        onClick = _this$props4.onClick,
        trigger = _this$props4.trigger,
        onMouseOver = _this$props4.onMouseOver,
        onMouseOut = _this$props4.onMouseOut,
        onFocus = _this$props4.onFocus,
        onBlur = _this$props4.onBlur,
        disabled = _this$props4.disabled;
    var triggerComponent = React.Children.only(children);
    var triggerProps = triggerComponent.props;
    var props = {
      key: 'triggerComponent',
      onClick: (0, _createChainedFunction.default)(triggerProps.onClick, onClick),
      'aria-describedby': (0, _get.default)(speaker, ['props', 'id'])
    };

    if (!disabled) {
      if ((0, _isOneOf.default)('click', trigger)) {
        props.onClick = (0, _createChainedFunction.default)(this.handleToggle, props.onClick);
      }

      if ((0, _isOneOf.default)('active', trigger)) {
        props.onClick = (0, _createChainedFunction.default)(this.show, props.onClick);
      }

      if ((0, _isOneOf.default)('hover', trigger)) {
        props.onMouseOver = (0, _createChainedFunction.default)(this.onMouseOverListener, triggerProps.onMouseOver, onMouseOver);
        props.onMouseOut = (0, _createChainedFunction.default)(this.onMouseOutListener, triggerProps.onMouseOut, onMouseOut);
      }

      if ((0, _isOneOf.default)('focus', trigger)) {
        props.onFocus = (0, _createChainedFunction.default)(this.handleDelayedShow, triggerProps.onFocus, onFocus);
        props.onBlur = (0, _createChainedFunction.default)(this.handleDelayedHide, triggerProps.onBlur, onBlur);
      }
    }

    return [React.cloneElement(triggerComponent, props), React.createElement(_Portal.default, {
      key: "portal"
    }, this.getOverlay())];
  };

  return OverlayTrigger;
}(React.Component);

OverlayTrigger.defaultProps = {
  trigger: ['hover', 'focus'],
  delayHide: 200,
  placement: 'bottomStart',
  rootClose: true
};
var _default = OverlayTrigger;
exports.default = _default;
module.exports = exports.default;